use std::{
	convert::TryFrom,
	error::Error,
	path::PathBuf,
	sync::Arc,
};

use tracing::{info};
use tracing_subscriber::{
	fmt, 
	fmt::format::FmtSpan,
	EnvFilter,
};

use ptth_relay::{
	Config,
	git_version::GIT_VERSION,
	RelayState,
	run_relay,
};

#[tokio::main]
async fn main () -> Result <(), Box <dyn Error>> {
	fmt ()
		.with_env_filter (EnvFilter::from_default_env ())
		.with_span_events (FmtSpan::FULL)
		.init ()
	;
	
	let config_path = PathBuf::from ("config/ptth_relay.toml");
	let config = Config::from_file (&config_path).await?;
	
	info! ("ptth_relay Git version: {:?}", GIT_VERSION);
	
	let (shutdown_rx, forced_shutdown) = ptth_core::graceful_shutdown::init_with_force ();
	
	forced_shutdown.wrap_server (
		run_relay (
			Arc::new (RelayState::try_from (config)?),
			shutdown_rx,
			Some (config_path)
		)
	).await??;
	
	Ok (())
}
